#!/home/moerman/cadbin/perl -w
# Turtle Graphics, C.M. Moerman, 25 oct 1999
#
# This program creates an interactive Perl/Tk window
# for playing with Perl and Tk commands in a turtle-graphics way.


#use strict;
#use diagnostics -verbose;

use Tk;				                    # use Tk GUI routines
use Tk::Dialog;			                # esp. Dialog box for display help
use Tk::Balloon;                        # and balloon help

$title = "Turtle Graphics, KM'99; file:";

my $top = MainWindow->new(-title => $title);

# ---------------- menu buttons ---------------

my $but_frame = $top->Frame();
   $but_frame->pack(-side => 'top', -anchor => "nw", -expand => 0, -fill => 'x', -padx => 8);
   
my $ball = $top->Balloon();             # balloon help

sub button {                            # create a menu-button
    my ($this, $text, $help, $command) = @_;
    my $button = $this->Button(-text => $text, -command => $command)->pack(-side => 'left');
    $ball->attach($button, -balloonmsg => $help);               # with attached help balloon
    $top->bind("<Alt-". substr(lc($text), 0, 1) . ">", $command );  # plus Alt-? shortcut (lower-case!)
    $button->configure(-underline => '0');                      # underline first char
    return $button;
}

#my $b_wisprog  = button($but_frame, "Erase","Erase drawing",       \&erase);
#my $b_newprog  = button($but_frame, "New",  "Erase all lines",     \&newprog);
#my $b_editprog = button($but_frame, "Load", "Load file",           \&loadprog);
#my $b_execprog = button($but_frame, "Run",  "Execute selected/all", \&execline);
#my $b_saveprog = button($but_frame, "Save", "Save lines to file",  \&saveprog);
#my $b_stopprog = button($but_frame, "Stop", "Stop your programm",  sub { $stop = 1 } );
#my $b_quitprog = button($but_frame, "Quit", "Quit TurtleGraphics", sub { $top->destroy; } );
#my $b_helpprog = button($but_frame, "Help", "Display help",        sub { $help_dialog->Show(); } );

my $b_wisprog  = button($but_frame, "Wis",   "Wis tekening",        \&erase);
my $b_newprog  = button($but_frame, "Nieuw", "Wis het programma",   \&newprog);
my $b_editprog = button($but_frame, "Laad",  "Laad een programma",  \&loadprog);
my $b_execprog = button($but_frame, "Doe het", "Voer het uit",      \&execline);
my $b_saveprog = button($but_frame, "Opslaan", "Sla programma op",  \&saveprog);
my $b_stopprog = button($but_frame, "Stop",  "Stop je programma",   sub { $stop = 1 } );
my $b_quitprog = button($but_frame, "Einde", "Beeindig TurtleGraphics",  sub { $top->destroy; } );
my $b_helpprog = button($but_frame, "Help",  "Laat de help zien",   sub { $help_dialog->Show(); } );

   $b_helpprog->pack(-side => 'right');

# ---------------- help box etc. -------------------------------

$help_dialog = $top->Dialog(-title => "Turtle Help", -default_button => "OK", -wraplength => '5i',
                            -text => <<HELP);
Turtle Graphics, Kees Moerman 1999

In het text-scherm kan je opdrachten intypen. Als je dan een opdracht selecteert kan je die met de 'Doe'-knop (of met Alt-d) uitvoeren.

Bestaande opdrachten zijn:
    vooruit (afstand);
    spring (afstand);
    links (hoek);
    rechts (hoek);
    huis
    wis
    als (test) { opdrachten } anders { opdrachten }
    voor (start .. end) { opdrachten }
    groen; geel; blauw; rood; wit; zwart; grijs;

Voorbeeld van opdrachten:

    vooruit 100;
    links; groen; vooruit 50;
    voor (1..10) { links 80; vooruit 100; }
HELP

$syntax_dialog = $top->Dialog(-title => "Turtle type-fout?", -text => "Nog niet....");

my $fail_dialog = $top->Dialog(-title => "Perl/Tk Interpreter Error", -default_button => "OK", -wraplength => '5i',
                            -text => "Laden of opslaan werkte niet: geen file gekozen?");
#                           -text => "Operation Load or Save did not succeed (no filename selected)");

# ---------------- text veld -------------------------------

$text = $top->Scrolled('Text',
	            -wrap => "word",
	            -scrollbars => "se",
	            -width => 40, -height => 8);
$text->pack(-expand => 1, -fill => 'both');
$text->focus();                         # gets focus on startup

my $filename = "Untitled";              # current file name
my %translation = (
    "voor"      => "foreach",   "vooruit"   => "forward",   "als"   => "if",        "anders" => "else",
    "links"     => "left",      "rechts"    => "right",     "huis"  => "home",      "wis"   => "erase",
    "green"     => "groen",     "blauw"     => "blue",      "rood"  => "red",       "geel"  => "yellow",
    "wit"       => "white",     "zwart"     => "black",     "grijs" => "gray",
    "spring"    => "jump");             # language translations dutch/english;
    
sub execline {
    $top->Busy;
	my $tags = $text->tagNextrange('sel', '1.0', 'end');
	$_ = $text->get('sel.first', 'sel.last') if defined $tags;
	$_ = $text->get('1.0', 'end') if !defined $tags;   # all if none selected

	foreach $word (keys %translation) { s/\b$word\b/$translation{$word}/g;  }
    s/\?/ print /g;
    s/\$arg(\d)\b/\$_[$1]/g;

    $stop = 0;
    $warn = "";                         # used to trap warning messages
	@result = eval;			            # evaluate entered expression

	if (!$stop && ($@ || $warn) && !($warn =~ "Subroutine .* redefined at"))
	{   $err = $@;
	    print "Warning = $warn\nError = $err";        # Oops, error
	    $syntax_dialog->configure(-text => "$warn$err");
        $syntax_dialog->Show();
    }
    else
	{
        print "\neval returned: " if !/^\s*print/;
	    print @result if !/^print/;	    # only print if statement didn't
	}
    $top->Unbusy;
}	

$SIG{__WARN__} = sub { $warn = $_[0]; };   # trap early warnings

sub newprog {                           # erase program
    $text->delete('1.0', 'end');
    $filename = "Untitled";
    $top->title("$title $filename");
}

sub loadprog {				            # read file into input pane
    my $line;
    my $filetypes = [ ['Turtle files', ['.tl']], ['All files', ['*']]];

    $filename =~ s/.*[\/\\]//;          # strip path
    $filename = $top->getOpenFile(-title => "Select file to open",
                    -initialfile => $filename, -filetypes => $filetypes, -defaultextension => ".tl");
    return $fail_dialog->Show() if ! -f $filename;

    $text->delete('1.0', 'end');        # first delete old contents
    if (open(FILE, $filename) || die "Can't open $filename")  # next read file line by line
    {   while($line = <FILE>)
        {   $text->insert('end', $line);
        }    
        close(FILE);
    }
    $top->title("$title $filename");
}

sub saveprog {				            # save input pane to file
    my $result;
    my $filetypes = [ ['Turtle files', ['.tl']]];
    
    $filename =~ s/.*[\/\\]//;          # strip path
    $filename = $top->getSaveFile(-title => "Select file to save to",
                    -initialfile => $filename, -filetypes => $filetypes, -defaultextension => ".tl");
    return $fail_dialog->Show() if !$filename;

    $result = $text->get('1.0', 'end');
    if(open(OUTPUT, ">$filename") || die "Can't open >$filename")
    {   print OUTPUT $result;           # save results to file in one go
        close(OUTPUT);
    }
    $top->title("$title $filename");
}

# ---------- Turtle graphics basics, created KM 9:55 24/10/99 -------------


use constant PI => 3.14159265358979323;
my ($xsize, $ysize) = (640, 440);       # +/- 200 plus borders

# create canvas for drawings
$canvas = $top->Canvas(-width => $xsize, -height => $ysize, -border => 1);
$canvas->configure(-relief => 'ridge', -background => 'white');
$canvas->pack();

my $color = "red";
my $width = 1;

sub degrees {   return $_[0] * PI / 180; }
sub dsin    {   return sin($_[0] * PI / 180); }
sub dcos    {   return cos($_[0] * PI / 180); }

sub xt      {   return $_[0] + $xsize/2; }
sub yt      {   return $ysize/2 - $_[0]; }


# redraw turtle at current pos
sub turtle
{
    $canvas->delete($t1) if defined $t1; 
    $canvas->delete($t2) if defined $t2; 
    $canvas->delete($t3) if defined $t3; 
    $canvas->delete($t4) if defined $t4; 
#   $t1 = $canvas->createLine(xt($xpos+10*dcos($dir+0)),   yt($ypos+10*dsin($dir+0)),   xt($xpos+ 5*dcos($dir+120)), yt($ypos+ 5*dsin($dir+120)), -fill => "green", -width => 2);
#   $t2 = $canvas->createLine(xt($xpos+ 5*dcos($dir+120)), yt($ypos+ 5*dsin($dir+120)), xt($xpos+ 5*dcos($dir-120)), yt($ypos+ 5*dsin($dir-120)), -fill => "green", -width => 2);
#   $t3 = $canvas->createLine(xt($xpos+ 5*dcos($dir-120)), yt($ypos+ 5*dsin($dir-120)), xt($xpos+10*dcos($dir+0)),   yt($ypos+10*dsin($dir+0)),   -fill => "green", -width => 2);

    $t1 = $canvas->createLine(xt($xpos+15*dcos($dir+45)), yt($ypos+15*dsin($dir+45)),  xt($xpos-15*dcos($dir+45)), yt($ypos-15*dsin($dir+45)), -fill => "dark green", -width => 2);
    $t2 = $canvas->createLine(xt($xpos+15*dcos($dir-45)), yt($ypos+15*dsin($dir-45)),  xt($xpos-15*dcos($dir-45)), yt($ypos-15*dsin($dir-45)), -fill => "dark green", -width => 2);
    $t3 = $canvas->createOval(xt($xpos+10), yt($ypos+10),  xt($xpos-10), yt($ypos-10), -fill => "green", -outline => "dark green", -width => 2);
    $t4 = $canvas->createOval(xt($xpos+12*dcos($dir)+4), yt($ypos+12*dsin($dir)+4),  xt($xpos+12*dcos($dir)-4), yt($ypos+12*dsin($dir)-4), -fill => "green", -outline => "dark green", -width => 2);

    if ($stop) { $warn = ""; die "Stop" }    # halted by user
    return "";
}


sub doforward     # go forward for x (default 20) steps
{
    my $step = $_[0] || 20;
    my $xnew = $xpos + dcos($dir) * $step;
    my $ynew = $ypos + dsin($dir) * $step;
    $canvas->createLine( xt($xpos), yt($ypos), xt($xnew), yt($ynew),
                        -fill => $color, -width => $width);
    $xpos = $xnew;
    $ypos = $ynew;
    turtle();
    $top->update;
#    sleep(1);
}

sub forward     # actual forward routine: break in steps
{               # to give impression of speed
    my $dist = $_[0] || 20;
    my $steps = 1 + int(abs($dist) / 10);
    foreach (1..$steps) { doforward $dist/$steps; }
}

sub jump        # go forward  without drawing for x (default 20) steps
{
    my $step = $_[0] || 20;
    $xpos = $xpos + dcos($dir) * $step;
    $ypos = $ypos + dsin($dir) * $step;
    turtle();
}

sub left    { $dir += $_[0] || 90;    turtle();  }    # turn left (degrees)
sub right   { left -($_[0] || 90); }
sub erase   { $canvas->delete('all'); home(); }
sub home    { ($xpos, $ypos, $dir) = (0, 0, 90);  turtle(); }   # reset at home

sub green   { $color = 'green'; }
sub blue    { $color = 'blue'; }
sub red     { $color = 'red'; }
sub yellow  { $color = 'yellow'; }
sub white   { $color = 'white'; }
sub black   { $color = 'black'; }
sub gray    { $color = 'gray'; }

home();         # initialise

# ------------------- main program ---------------------------


MainLoop();				# start 'program'

#end of Perl script
